﻿namespace MurdocBatch
{
    using System;
    using System.Collections.Generic;

    class MurdocBatch
    {
        static void Main(string[] args)
        {
            try
            {
                ArgAnalyze analyze = ArgAnalyze.Analyze(args);

                if (!System.IO.File.Exists(analyze.MurdocCutterPath))
                {
                    Console.WriteLine("*** MurdocCutterが見つかりません ***");
                    System.Environment.ExitCode = 2;
                    return;
                }

                if (!System.IO.Path.GetExtension(analyze.MurdocCutterPath).Equals(".exe" , System.StringComparison.OrdinalIgnoreCase ))
                {
                    Console.WriteLine("*** MurdocCutterのパスがexeではありません ***");
                    System.Environment.ExitCode = 2;
                    return;
                }

                System.Diagnostics.FileVersionInfo fvi = System.Diagnostics.FileVersionInfo.GetVersionInfo(analyze.MurdocCutterPath);
                System.Version ver = new System.Version(fvi.FileMajorPart,fvi.FileMinorPart,fvi.FileBuildPart,fvi.FilePrivatePart );
                if (fvi.ProductName != "Murdoc Cutter")
                {
                    Console.WriteLine("*** MurdocCutterと認識できませんでした ***");
                    System.Environment.ExitCode = 2;
                    return;
                }

                if (ver < new Version(1,9,8,1))
                {
                    Console.WriteLine("*** MurdocCutterのバージョンが古いです ***");
                    System.Environment.ExitCode = 2;
                    return;
                }


                Batches batches = Batches.ReadBatchFile(analyze.BatchPath);

                System.Diagnostics.ProcessStartInfo pinfo
                    = new System.Diagnostics.ProcessStartInfo(analyze.MurdocCutterPath);

                System.Diagnostics.Process process
                    = new System.Diagnostics.Process();
                process.StartInfo = pinfo;

                var x = batches.ClipListNotExists;
                if(x.Count>0 )
                {
                    Console.WriteLine("*** ClipListが見つかりません ***");
                    foreach (string s in x)
                    {
                        Console.WriteLine(s);
                    }
                    System.Environment.ExitCode = 2;
                    return;
                }

                var y = batches.OutputExists;
                if(y.Count>0)
                {
                    Console.WriteLine("*** 出力先が既に存在しているためキャンセルしました ***");
                    foreach (string s in y)
                    {
                        Console.WriteLine(s);
                    }
                    System.Environment.ExitCode = 2;
                    return;
                }
                var z = batches.HasDoubleOutputPath;
                if (z.Count > 0)
                {
                    Console.WriteLine("*** 出力先が重複しているためキャンセルしました ***");
                    foreach (string s in z)
                    {
                        Console.WriteLine(s);
                    }
                    System.Environment.ExitCode = 2;
                    return;
                }

                if (batches.List.Count > 0)
                {
                    if (analyze.StartTime > DateTime.Now)
                    {
                        Console.WriteLine(analyze.StartTime.ToString() + "まで待機します");
                        while (analyze.StartTime > DateTime.Now)
                        {
                            Console.SetCursorPosition(0, Console.CursorTop );
                            Console.Write( DateTime.Now.ToString());
                            System.Threading.Thread.Sleep(1000);
                        }
                        Console.WriteLine();
                    }

                    foreach (Batch batch in batches.List)
                    {
                        Console.WriteLine(batch.OutputPath);
                        if (!analyze.isTest)
                        {
                            batch.RewriteOutput_And_CopyClipList(analyze);

                            process.Start();
                            process.WaitForInputIdle();

                            IntPtr hMurdoc = process.MainWindowHandle;
                            IntPtr hChild = IntPtr.Zero;
                            hChild = API.FindWindowEx(hMurdoc , hChild , "Button" , analyze.ClippingButtonText);

                            if (hChild != IntPtr.Zero)
                            {
                                Console.WriteLine(" -> " + batch.OutputPath);
                                API.SendMessage(hChild , (uint)API.WM.WM_LBUTTONDOWN , IntPtr.Zero , IntPtr.Zero);
                                API.SendMessage(hChild , (uint)API.WM.WM_LBUTTONUP , IntPtr.Zero , IntPtr.Zero);

                                do
                                {
                                    System.Threading.Thread.Sleep(1000);

                                } while (API.GetText(hChild) != analyze.ClippingButtonText  || !API.IsWindowEnabled(hChild));
                            }
                            System.Threading.Thread.Sleep(1000);
                            API.SendMessage(hMurdoc , (uint)API.WM.WM_CLOSE , IntPtr.Zero , IntPtr.Zero);
                            if (!process.WaitForExit(10000))
                            {
                                Console.WriteLine("閉じることが出来ませんでした");
                                System.Environment.ExitCode = 4;
                                break;
                            }
                            if (hChild == IntPtr.Zero)
                            {
                                Console.WriteLine("Clippingボタンが見つかりませんでした");
                                System.Environment.ExitCode = 3;
                                break;
                            }
                        }
                    }
                }
                Console.WriteLine("正常に終了");
                System.Environment.ExitCode = 0;
                if (analyze.PowerAtFinish != ArgAnalyze.Power.None )
                {
                    API.SetSuspendState(analyze.PowerAtFinish== ArgAnalyze.Power.Sleep  , false , false);
                }

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                System.Environment.ExitCode = 1;
            }

        }
        
    }

    class ArgAnalyze
    {
        public enum Power
        {
            None = 0 ,Sleep , Standby
        }

        public string ClippingButtonText = "Clpping";
        public string MurdocCutterPath = "MurdocCut.exe";
        public string MurdocInitPath = "MurdocCut.ini";
        public string BatchPath = System.Environment.CurrentDirectory;
        public bool isTest = false;

        public DateTime StartTime = DateTime.MinValue;
        public Power PowerAtFinish = Power.None;

        public static ArgAnalyze Analyze(string[] args)
        {
            ArgAnalyze aa=new ArgAnalyze();
            int i = 0;
            while (i < args.Length)
            {
                if (args[i][0] == '/')
                {
                    switch (char.ToUpper( args[i][1]))
                    {
                    case 'M':
                        i++;
                        aa.MurdocCutterPath = args[i];
                        aa.MurdocInitPath = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(aa.MurdocCutterPath) , "MurdocCut.ini");
                        break;

                    case 'B':
                        i++;
                        aa.BatchPath = args[i];
                        break;
                    case 'T':
                        aa.isTest = true;
                        break;
                    case 'D':
                        i++;
                        aa.StartTime = DateTime.Parse(args[i]);
                        break;
                    case 'S':
                        if (args[i].Length > 2)
                        {
                            switch (char.ToUpper(args[i][2]))
                            {
                            case 'L':
                                aa.PowerAtFinish = Power.Sleep;
                                break;
                            case 'T':
                                aa.PowerAtFinish = Power.Standby;
                                break;

                            }
                        }
                        break;
                    case 'C':
                        i++;
                        aa.ClippingButtonText = args[i];
                        break;
                    }
                }
                i++;
            }
            return aa;
        }
    }

    class Batch
    {
        public string ClipListPath;
        public string OutputPath;

        public void RewriteOutputPath(string iniPath)
        {
            string all;
            using (System.IO.StreamReader sr = new System.IO.StreamReader(iniPath,System.Text.Encoding.Default))
            {
                all=sr.ReadToEnd();
            }
            using (System.IO.StringReader sr = new System.IO.StringReader(all))
            {
                bool isFile = false;
                using (System.IO.StreamWriter sw = new System.IO.StreamWriter(iniPath ,false, System.Text.Encoding.Default))
                {
                    while (sr.Peek() != -1)
                    {
                        string line;
                        line = sr.ReadLine();

                        if (line.StartsWith("["))
                        {
                            isFile = (line=="[FILE]");
                        }
                        if (isFile && line.StartsWith("OUTPUT="))
                        {
                            sw.WriteLine("OUTPUT=" + OutputPath);
                        }
                        else
                        {
                            sw.WriteLine(line);
                        }
                    }
                }
            }
        }

        public void RewriteOutput_And_CopyClipList(ArgAnalyze aa)
        {
            RewriteOutputPath(aa.MurdocInitPath);

            string binPath 
                = System.IO.Path.Combine(
                    System.IO.Path.GetDirectoryName(aa.MurdocInitPath)
                    ,"Clip.bin");
            CopyClipList(binPath);
        }

        public void CopyClipList(string originalBinPath)
        {
            System.IO.File.Copy(this.ClipListPath , originalBinPath,true);
        }
    }

    class Batches
    {
        private Batches()
        {
        }

        public List<Batch> List
        {
            get { return _List; }
        }
        private List<Batch> _List = new List<Batch>();

        public static Batches ReadBatchFile(string batchPath)
        {
            Batches bats = new Batches();

            batchPath = batchPath.Trim();

            if (System.IO.Directory.Exists(batchPath))
            {
                foreach (string path in System.IO.Directory.GetFiles(batchPath , "*.bin" , System.IO.SearchOption.TopDirectoryOnly))
                {
                    Batch bat = new Batch();
                    bat.ClipListPath = path;
                    bat.OutputPath = System.IO.Path.Combine(System.IO.Path.GetDirectoryName(path)
                                                          , System.IO.Path.GetFileNameWithoutExtension(path) + ".ts");
                    bats.List.Add(bat);
                }
            }
            else if (System.IO.File.Exists(batchPath))
            {
                using (System.IO.StreamReader sr = new System.IO.StreamReader(batchPath , System.Text.Encoding.Default))
                {
                    while (sr.Peek() != -1)
                    {
                        string line;
                        line = sr.ReadLine();
                        string[] datas = line.Split('|');
                        for (int i = 0; i < datas.Length; i++)
                        {
                            datas[i] = datas[i].Trim();
                            if (datas[i].StartsWith("\"") && datas[i].EndsWith("\""))
                            {
                                datas[i] = datas[i].Substring(1 , datas[i].Length - 2);
                            }
                        }
                        if (datas.Length >= 2)
                        {
                            Batch bat = new Batch();
                            bat.ClipListPath = datas[0];
                            bat.OutputPath = datas[1];


                            bats.List.Add(bat);
                        }
                    }
                }
            }
            return bats;
        }

        public List<string> ClipListNotExists
        {
            get
            {
                List<string> retval = new List<string>();
                foreach (Batch batch in this.List)
                {
                    if (!System.IO.File.Exists(batch.ClipListPath))
                    {
                        retval.Add(batch.ClipListPath);
                    }
                }
                return retval;
            }
        }


        public List<string> OutputExists
        {
            get
            {
                List<string> retval = new List<string>();
                foreach (Batch batch in this.List)
                {
                    if (System.IO.File.Exists(batch.OutputPath))
                    {
                        retval.Add(batch.OutputPath);
                    }
                }
                return retval;
            }
        }

        public List<string> HasDoubleOutputPath
        {
            get
            {

                System.Collections.Generic.List<string> list = new List<string>();
                System.Collections.Generic.List<string> retval = new List<string>();
                foreach (Batch batch in List)
                {
                    if (list.Contains(batch.OutputPath))
                    {
                        retval.Add(batch.OutputPath);
                    }
                    list.Add(batch.OutputPath);
                }

                return retval;
            }
        }
        public void DeleteExists()
        {
            foreach (Batch batch in this.List)
            {
                if (System.IO.File.Exists(batch.OutputPath))
                {
                    System.IO.File.Delete(batch.OutputPath);
                }
            }
        }
    }
}

namespace MurdocBatch
{
    using System;
    using System.Runtime.InteropServices;
 
    class API
    {
        [DllImport("user32.dll" , SetLastError = true)]
        public static extern IntPtr FindWindowEx(IntPtr hwndParent , IntPtr hwndChildAfter , string lpszClass , string lpszWindow);

        [DllImport("user32.dll" , SetLastError = true)]
        public static extern IntPtr FindWindowEx(IntPtr parentHandle , IntPtr childAfter , string className , IntPtr windowTitle);

        [DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessage(IntPtr hWnd , UInt32 Msg , IntPtr wParam , IntPtr lParam);
        [DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessage(IntPtr hWnd , UInt32 Msg , int wParam, System.Text.StringBuilder lParam);

        [DllImport("user32.dll" , CharSet = CharSet.Auto)]
        public static extern IntPtr PostMessage(IntPtr hWnd , UInt32 Msg , IntPtr wParam , IntPtr lParam);

        
        [DllImport("user32.dll")]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool IsWindowEnabled(IntPtr hWnd);

        [System.Runtime.InteropServices.DllImport("Powrprof.dll" , SetLastError = true)]
        public static extern bool SetSuspendState(bool hibernate , bool forceCritical , bool disableWakeEvent);


        public static string GetText(IntPtr hwnd)
        {
            int size;

            size = SendMessage(hwnd , (uint)WM.WM_GETTEXTLENGTH , IntPtr.Zero  , IntPtr.Zero ).ToInt32();
            System.Text.StringBuilder sb = new System.Text.StringBuilder(size + 1);
            SendMessage(hwnd , (uint)WM.WM_GETTEXT , size + 1 , sb);
            return sb.ToString();
        }

        public enum WM : uint
        {
            WM_GETTEXTLENGTH = 0x000E,
            WM_GETTEXT = 0x000D,
            WM_CLOSE = 0x10 ,
            WM_LBUTTONDOWN = 0x0201 ,
            WM_LBUTTONUP = 0x0202
            
        }
    }
}
